@@ -29,6 +29,16 @@ var deleteOperatorScenarios = []expressionScenario{
 			"D0, P[a], (!!map)::{a2: frood}\n",
 		},
 	},
+	{
+		skipDoc:     true,
+		description: "delete whole document",
+		document2:   `a: slow`,
+		document:    `a: fast`,
+		expression:  `del(select(.a == "fast"))`,
+		expected: []string{
+			"D0, P[], (doc)::a: slow\n",
+		},
+	},
 	{
 		skipDoc:    true,
 		document:   `a: [1,2,3]`,
@@ -171,5 +181,5 @@ func TestDeleteOperatorScenarios(t *testing.T) {
 	for _, tt := range deleteOperatorScenarios {
 		testScenario(t, &tt)
 	}
-	documentScenarios(t, "delete", deleteOperatorScenarios)
+	documentOperatorScenarios(t, "delete", deleteOperatorScenarios)
 }
