@@ -3,7 +3,7 @@ package yqlib
 func splitDocumentOperator(d *dataTreeNavigator, context Context, expressionNode *ExpressionNode) (Context, error) {
 	log.Debugf("-- splitDocumentOperator")
 
-	var index uint = 0
+	var index uint
 	for el := context.MatchingNodes.Front(); el != nil; el = el.Next() {
 		candidate := el.Value.(*CandidateNode)
 		candidate.Document = index
